import 'package:tencent_cloud_chat/components/component_config/tencent_cloud_chat_message_config.dart';
import 'package:tencent_cloud_chat/components/component_config/tencent_cloud_chat_user_config.dart';
import 'package:tencent_cloud_chat/components/tencent_cloud_chat_components_utils.dart';
import 'package:tencent_cloud_chat/data/tencent_cloud_chat_data_abstract.dart';
import 'package:tencent_cloud_chat/models/tencent_cloud_chat_models.dart';
import 'package:tencent_cloud_chat/tencent_cloud_chat.dart';
import 'package:tencent_cloud_chat_sdk/tencent_im_sdk_plugin.dart';

/// An enumeration of basic data keys for TencentCloudChat Core.
enum TencentCloudChatBasicDataKeys {
  none,
  hasInitialized,
  usedComponents,
  hasLoggedIn,
  selfInfo
}

/// A class that manages the basic data for TencentCloudChat Core.
///
/// This class extends [TencentCloudChatUIKitCoreDataAB] and provides
/// functionality for managing the initialization status of the UIKit Core.
class TencentCloudChatBasicData<T> extends TencentCloudChatDataAB<T> {
  TencentCloudChatBasicData(super.currentUpdatedFields);

  /// ==== useCallKit ====
  static bool _useCallKit = false;

  // ignore: unnecessary_getters_setters
  bool get useCallKit => _useCallKit;

  set useCallKit(bool value) {
    _useCallKit = value;
  }

  double? get keyboardHeight {
    return TencentCloudChat.cache.getCurrentDeviceKeyBordHeight();
  }

  set keyboardHeight(double? value) {
    if (value != null) {
      TencentCloudChat.cache.cacheCurrentDeviceKeyBordHeight(value);
    }
  }

  /// ==== hasInitialized ====

  static bool _hasInitialized = false;

  static bool _hasLoggedIn = false;

  static int _sdkappid = 0;

  static List<
      ({
        Map<String, dynamic> pluginData,
        TencentCloudChatPlugin pluginInstance,
        TencentCloudChatPluginNameEnum pluginName,
        TencentCloudChatPluginWidget? builder
      })> plugins = [];

  int get sdkappid => _sdkappid;

  bool get hasLoggedIn => _hasLoggedIn;

  bool get hasInitialized => _hasInitialized;

  static final TencentCloudChatUserConfig _userConfig =
      TencentCloudChatUserConfig(
    useUserOnlineStatus: true,
    autoDownloadMultimediaMessage: false,
  );

  /// ==== userConfig ====

  TencentCloudChatUserConfig get userConfig => _userConfig;

  addPlugin(
      ({
        Map<String, dynamic> pluginData,
        TencentCloudChatPlugin pluginInstance,
        TencentCloudChatPluginNameEnum pluginName,
        TencentCloudChatPluginWidget? builder
      }) plugin) {
    plugins.add(plugin);
  }

  hasPlugins(TencentCloudChatPluginNameEnum name) {
    return plugins.indexWhere((element) => element.pluginName == name) > -1;
  }

  ({
    Map<String, dynamic> pluginData,
    TencentCloudChatPlugin pluginInstance,
    TencentCloudChatPluginNameEnum pluginName,
    TencentCloudChatPluginWidget? builder
  })? getPlugin(TencentCloudChatPluginNameEnum name) {
    if (hasPlugins(name)) {
      int idx = plugins.indexWhere((element) => element.pluginName == name);
      return plugins[idx];
    }
    return null;
  }

  updateUseUserOnlineStatus(TencentCloudChatUserConfig config) {
    if (config.autoDownloadMultimediaMessage != null) {
      _userConfig.autoDownloadMultimediaMessage =
          config.autoDownloadMultimediaMessage;
    }
    if (config.useUserOnlineStatus != null) {
      _userConfig.useUserOnlineStatus = config.useUserOnlineStatus;
    }
  }

  updateSDKAppID({
    required int sdkappid,
  }) {
    _sdkappid = sdkappid;
  }

  bool updateLoginStatus({
    required bool status,
  }) {
    _hasLoggedIn = status;

    console(logs: "Login Status Changed. Current Status Is $status");

    notifyListener(TencentCloudChatBasicDataKeys.hasLoggedIn as T);

    return _hasLoggedIn;
  }

  /// Updates the initialization status of the UIKit Core.
  ///
  /// The [status] parameter indicates the new initialization status.
  /// Returns the updated initialization status.
  bool updateInitializedStatus({
    required bool status,
  }) {
    _hasInitialized = status;

    console(logs: "Init Status Changed. Current Status Is $status");

    notifyListener(TencentCloudChatBasicDataKeys.hasInitialized as T);

    return _hasInitialized;
  }

  /// ==== Current User ====
  static V2TimUserFullInfo? _currentUser;
  V2TimUserFullInfo? get currentUser {
    return TencentCloudChat.cache.getCurrentLoginUserInfo() ?? _currentUser;
  }

  void updateCurrentUserInfo({required V2TimUserFullInfo userFullInfo}) {
    _currentUser = userFullInfo;
    TencentCloudChat.cache.cacheCurrentLoginUserInfo(userFullInfo);
    notifyListener(TencentCloudChatBasicDataKeys.selfInfo as T);
  }

  /// ==== version ====
  static String _version = "";
  String get version => _version;
  void updateVersion({required String version}) {
    _version = version;
  }

  /// ==== usedComponents ====

  static List<TencentCloudChatComponentsEnum> _usedComponents = [];

  List<TencentCloudChatComponentsEnum> get usedComponents => _usedComponents;

  set usedComponents(List<TencentCloudChatComponentsEnum> value) {
    _usedComponents = value;

    console(
        logs:
            "`usedComponents` Status Changed. Current `usedComponents` Is $_usedComponents");

    notifyListener(TencentCloudChatBasicDataKeys.usedComponents as T);
  }

  void addUsedComponent(TencentCloudChatComponentsEnum componentName) {
    _usedComponents.add(componentName);
    _usedComponents.toSet().toList();
  }

  /// ==== config for components ====
  static TencentCloudChatMessageConfig? _messageConfig;

  TencentCloudChatMessageConfig get messageConfig =>
      _messageConfig ?? TencentCloudChatMessageConfig();

  set messageConfig(TencentCloudChatMessageConfig value) {
    _messageConfig = value;
  }

  @override
  Map<String, dynamic> toJson() {
    return Map<String, dynamic>.from({
      "hasInitialized": _hasInitialized,
    });
  }

  @override
  void notifyListener(T key) {
    currentUpdatedFields = key;
    TencentCloudChat.eventBusInstance.fire(this);
  }
}
